package com.bero.multimedia.beroxm;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.lang.reflect.Field;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.ShortBuffer;
import java.util.Timer;
import java.util.TimerTask;

import java.util.concurrent.locks.ReentrantLock;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.media.MediaPlayer;
import android.os.IBinder;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.Menu;
import android.os.Binder;
import android.os.PowerManager;
import android.os.Process;
import android.os.RemoteCallbackList;
import android.os.RemoteException;
import android.os.SystemClock;

@SuppressWarnings("unused")
public class BeRoXMService extends Service{
	public final static String TAG="BeRoXMService";

	private final static int PlayerStateShutdown=0;
	private final static int PlayerStateIdle=1;
	private final static int PlayerStatePlaying=2;
	private final static int PlayerStatePaused=3;

	private PowerManager.WakeLock mWakeLock=null;

	public AtomicInteger mPlayerState=new AtomicInteger(PlayerStateIdle);

	final RemoteCallbackList<IBeRoXMServiceCallback> mCallbacks=new RemoteCallbackList<IBeRoXMServiceCallback>();

	public BeRoXM mBeRoXM=null;

	public AtomicLong mPosition=new AtomicLong();
	public long mDuration=0;
	public AtomicLong mNewPosition=new AtomicLong();
	public AtomicBoolean mNewTrack=new AtomicBoolean();

	private String mTitle="";

	private Boolean Looping=true;

	private Thread updateThread,audioThread;

	private Object mSyncObj=new Object();
	private Object mBroadcastSyncObj=new Object();

	public BeRoXMService(){
		super();
	}

	@Override
	public void onCreate(){
		Log.d(TAG,"Entering onCreate");
		super.onCreate();

		mPlayerState.set(PlayerStateIdle);

		mNewTrack.set(false);

		if(mWakeLock==null){
			PowerManager pm=(PowerManager)getSystemService(Context.POWER_SERVICE);
			mWakeLock=pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,this.getClass().getName());
			mWakeLock.setReferenceCounted(false);
		}

		mBeRoXM=new BeRoXM();
		mBeRoXM.nativeInit();

		updateThread=new Thread(null,updateTask,TAG);
		updateThread.start();

		audioThread=new Thread(null,AudioTask,TAG);
		audioThread.start();

		Log.d(TAG,"Leaving onCreate");
	}

	@Override
	public void onDestroy(){
		Log.d(TAG,"Entering onDestroy");
		synchronized(mSyncObj){
			mPlayerState.set(PlayerStateShutdown);

			if(mBeRoXM!=null){
				mBeRoXM.nativeDone();
				mBeRoXM=null;
			}
		}

		if((mWakeLock!=null)&&mWakeLock.isHeld()){
			mWakeLock.release();
		}

		try{
			for(int i=10;(i>0)&&(updateThread.isAlive()||audioThread.isAlive());i--){
				Thread.sleep(100);
			}
		}catch(InterruptedException e){
			e.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}

		super.onDestroy();
		Log.d(TAG,"Leaving onDestroy");
	}

	public boolean BeRoXMLoad(final String fileName){
		boolean b=false;
		if((mBeRoXM!=null)&&(mPlayerState.get()!=PlayerStateShutdown)){
			synchronized(mSyncObj){
				mPlayerState.set(PlayerStateIdle);
				if(mBeRoXM.nativeLoad(fileName)){
					mNewTrack.set(true);
					mBeRoXM.nativeSetLooping(Looping);
					mPosition.set(0);
					mDuration=mBeRoXM.nativeGetDuration();
					mTitle=mBeRoXM.nativeGetTitle();
					if((mTitle==null)||(mTitle.trim().equals(""))){
						mTitle=(new File(fileName)).getName();
					}
					b=true;
				}else{
					mPosition.set(0);
					mDuration=0;
					mTitle="";
				}
				synchronized(mBroadcastSyncObj){
					final int j=mCallbacks.beginBroadcast();
					for(int i=0;i<j;i++){
						try{
							IBeRoXMServiceCallback cb=mCallbacks.getBroadcastItem(i);
							cb.positionChanged(mPosition.get()/44100);
							cb.durationChanged(mDuration);
							cb.titleChanged(mTitle);
						}catch(Exception e){
							Log.e(TAG,"Error dispatching messages to clients",e);
						}
					}
					mCallbacks.finishBroadcast();
				}
			}
		}
		if(b){
			mPlayerState.set(PlayerStatePaused);
		}
		return b;
	}

	public long BeRoXMSeek(long toPosition){
		synchronized(mSyncObj){
			final int playerState=mPlayerState.get();
			if((mBeRoXM!=null)&&((playerState==PlayerStatePlaying)||(playerState==PlayerStatePaused))){
				return mBeRoXM.nativeSeek(toPosition);
			}
		}
		return 0;
	}

	public boolean BeRoXMPlay(){
		synchronized(mSyncObj){
			if((mBeRoXM!=null)&&(mBeRoXM.nativePlay())){
				mBeRoXM.nativeSetLooping(Looping);
				mPlayerState.set(PlayerStatePlaying);
				if(!mWakeLock.isHeld()){
					mWakeLock.acquire();
				}
				return true;
			}else{
				mPlayerState.set(PlayerStateIdle);
				return false;
			}
		}
	}

	public boolean BeRoXMStop(){
		synchronized(mSyncObj){
			final int playerState=mPlayerState.get();
			if((mBeRoXM!=null)&&((playerState==PlayerStatePlaying)||(playerState==PlayerStatePaused))){
				mPlayerState.set(PlayerStateIdle);
				if(mWakeLock.isHeld()){
					mWakeLock.release();
				}
				mPosition.set(0);
				mDuration=0;
				mTitle="";
				synchronized(mBroadcastSyncObj){
					final int j=mCallbacks.beginBroadcast();
					for(int i=0;i<j;i++){
						try{
							IBeRoXMServiceCallback cb=mCallbacks.getBroadcastItem(i);
							cb.positionChanged(mPosition.get()/44100);
							cb.durationChanged(mDuration);
							cb.titleChanged(mTitle);
						}catch(Exception e){
							Log.e(TAG,"Error dispatching messages to clients",e);
						}
					}
					mCallbacks.finishBroadcast();
				}
				return mBeRoXM.nativeStop();
			}else{
				return false;
			}
		}
	}

	public boolean BeRoXMPause(){
		final int playerState=mPlayerState.get();
		if((mBeRoXM!=null)&&((playerState==PlayerStatePlaying)||(playerState==PlayerStatePaused))){
			mPlayerState.set(PlayerStatePaused);
			if(mWakeLock.isHeld()){
				mWakeLock.release();
			}
			return true;
		}else{
			return false;
		}
	}

	public boolean BeRoXMResume(){
		final int playerState=mPlayerState.get();
		if((mBeRoXM!=null)&&((playerState==PlayerStatePlaying)||(playerState==PlayerStatePaused))){
			mPlayerState.set(PlayerStatePlaying);
			if(!mWakeLock.isHeld()){
				mWakeLock.acquire();
			}
			return true;
		}else{
			return false;
		}
	}

	private final IBeRoXMService.Stub mBinder=new IBeRoXMService.Stub(){

		@Override
		public boolean load(String fileName) throws RemoteException{
			return BeRoXMLoad(fileName);
		}

		@Override
		public boolean pause() throws RemoteException{
			return BeRoXMPause();
		}

		@Override
		public boolean play() throws RemoteException{
			return BeRoXMPlay();
		}

		@Override
		public boolean resume() throws RemoteException{
			return BeRoXMResume();
		}

		@Override
		public long seek(long toPosition) throws RemoteException{
			mNewPosition.set(toPosition);
			mPosition.set(toPosition);
			return toPosition;
		}

		@Override
		public boolean stop() throws RemoteException{
			return BeRoXMStop();
		}

		@Override
		public void registerCallback(IBeRoXMServiceCallback cb) throws RemoteException{
			if(cb!=null){
				synchronized(mSyncObj){
					synchronized(mBroadcastSyncObj){
						mCallbacks.register(cb);
						cb.positionChanged(mPosition.get()/44100);
						cb.durationChanged(mDuration);
						cb.titleChanged(mTitle);
					}
				}
			}
		}

		@Override
		public void unregisterCallback(IBeRoXMServiceCallback cb) throws RemoteException{
			if(cb!=null){
				synchronized(mSyncObj){
					synchronized(mBroadcastSyncObj){
						mCallbacks.unregister(cb);
					}
				}
			}
		}

		@Override
		public void fillBuffer(byte[] buffer,int bytes) throws RemoteException{
		}

		@Override
		public void setLooping(boolean looping) throws RemoteException{
			Looping=looping;
		}

		@Override
		public String getTitle() throws RemoteException{
			return mTitle;
		}

		@Override
		public boolean loaded() throws RemoteException{
			final int playerState=mPlayerState.get();
			return (playerState==PlayerStatePlaying)||(playerState==PlayerStatePaused);
		}

		@Override
		public boolean paused() throws RemoteException{
			final int playerState=mPlayerState.get();
			return playerState==PlayerStatePaused;
		}

	};

	@Override
	public IBinder onBind(Intent intent){
		return mBinder;
	}

	Runnable updateTask=new Runnable(){
		public void run(){
			try{
				Object broadcastSyncObj=mBroadcastSyncObj;
				long lastPosition=-1;
				boolean OK=false;
				while(mPlayerState.get()!=PlayerStateShutdown){
					final long newPosition=mPosition.get()/44100;
					if(lastPosition!=newPosition){
						lastPosition=newPosition;
						synchronized(broadcastSyncObj){
							try{
								OK=true;
								final int j=mCallbacks.beginBroadcast();
								for(int i=0;i<j;i++){
									try{
										IBeRoXMServiceCallback cb=mCallbacks.getBroadcastItem(i);
										cb.positionChanged(lastPosition);
									}catch(Exception e){
										Log.e(TAG,"Error dispatching messages to clients",e);
										OK=false;
									}
								}
								mCallbacks.finishBroadcast();
							}catch(Exception e){
								Log.e(TAG,"Error beginning to dispatch callback messages to clients",e);
								OK=false;
							}
						}
					}
					Thread.sleep(OK?250:1000);
				}
			}catch(Exception e){
				Log.e(TAG,"Error in update track thread.",e);
			}
		}
	};

	Runnable AudioTask=new Runnable(){
		public void run(){
			Log.d(TAG,"Starting audio track thread");
			Object syncObj=mSyncObj;
			int minBufferSizeInBytes=AudioTrack.getMinBufferSize(44100,AudioFormat.CHANNEL_CONFIGURATION_STEREO,AudioFormat.ENCODING_PCM_16BIT);
			int bufferSizeInBytes=Math.max(88200/*500ms*/,minBufferSizeInBytes);
			int sleepTime=Math.min(100,Math.max(10,(int)(((bufferSizeInBytes>>2)*1000)/44100)>>2));
			byte[] byteArray=new byte[bufferSizeInBytes];
			ByteBuffer byteBuffer=ByteBuffer.allocateDirect(byteArray.length).order(ByteOrder.nativeOrder());
			AudioTrack track=new AudioTrack(AudioManager.STREAM_MUSIC,44100,AudioFormat.CHANNEL_CONFIGURATION_STEREO,AudioFormat.ENCODING_PCM_16BIT,bufferSizeInBytes,AudioTrack.MODE_STREAM);
			int offset=0;
			int bufferLength=byteArray.length;
			int updateCount=0;
			long newPosition;
			long timeOffset=0;
			long position=0;
			boolean wasPaused=true;
			boolean oldLooping=Looping;
			boolean useHeadPos=android.os.Build.VERSION.SDK_INT>=android.os.Build.VERSION_CODES.ECLAIR;
			Process.setThreadPriority(Process.THREAD_PRIORITY_AUDIO);
			if(!mWakeLock.isHeld()){
				mWakeLock.acquire();
			}
			try{
				int playerState;
				while((playerState=mPlayerState.get())!=PlayerStateShutdown){
					if(playerState==PlayerStatePlaying){
						newPosition=mNewPosition.getAndSet(-1);
						if(mNewTrack.getAndSet(false)){
							if(!wasPaused){
								track.stop();
								wasPaused=true;
							}
							position=0;
						}
						if(wasPaused||(newPosition>=0)||(offset>=byteBuffer.capacity())){
							if(newPosition>=0){
								synchronized(syncObj){
									if(mBeRoXM!=null){
										mBeRoXM.nativeStop();
										mBeRoXM.nativePlay();
									}
								}
								position=BeRoXMSeek(newPosition);
								// / position=BeRoXMSeek(Math.max(0,Math.min(newPosition,mDuration-22050)));
								mPosition.set(position);
								if(!wasPaused){
									track.stop();
								}
								track.play();
								if(useHeadPos&&(position>=0)){
									timeOffset=position-track.getPlaybackHeadPosition();
								}
							}else if(wasPaused){
								track.play();
								if(useHeadPos&&(position>=0)){
									timeOffset=position-track.getPlaybackHeadPosition();
								}
							}
							wasPaused=false;
							byteBuffer.position(0);
							synchronized(syncObj){
								if(mBeRoXM!=null){
									if(oldLooping!=Looping){
										oldLooping=Looping;
										mBeRoXM.nativeSetLooping(oldLooping);
									}
									position=mBeRoXM.nativeGetPosition();
									mBeRoXM.nativeFillBuffer(byteBuffer,bufferLength);
								}
							}
							byteBuffer.position(0);
							byteBuffer.get(byteArray);
							offset=0;
							if(useHeadPos&&(position>=0)){
								timeOffset=position-track.getPlaybackHeadPosition();
							}
						}
						mPosition.set(useHeadPos?(track.getPlaybackHeadPosition()+timeOffset):position);
						int len=bufferLength-offset;
						int bytes=track.write(byteArray,offset,len);
						if(bytes<0){
							break;
						}else if(bytes!=len){
							Thread.sleep(sleepTime);
						}
						position+=bytes>>2;
						offset+=bytes;
						updateCount+=bytes;
					}else{
						if(!wasPaused){
							track.stop();
							wasPaused=true;
						}
						Thread.sleep(100);
					}
					if(updateCount>176400){
						updateCount-=176400;
					}
				}
			}catch(Exception e){
				Log.e(TAG,"Error in audio track thread.",e);
			}
			if(!wasPaused){
				track.stop();
			}
			track.release();
			if(mWakeLock.isHeld()){
				mWakeLock.release();
			}
			Log.d(TAG,"Stopping audio track thread");
		}
	};

}
