package com.bero.multimedia.beroxm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.io.BufferedReader;
import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.AlphaAnimation;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.content.DialogInterface.OnClickListener;
import android.content.pm.PackageManager;

import com.admob.android.ads.AdView;
import com.admob.android.ads.AdManager;

public class MainActivity extends Activity implements Comparator<File>{
	public final static String TAG="BeRoXM";

	private static final int MENU_LOOPING=Menu.FIRST+1;
	private static final int MENU_REFRESH=Menu.FIRST+2;
	private static final int MENU_BUY=Menu.FIRST+3;
	private static final int MENU_ABOUT=Menu.FIRST+4;
	private static final int MENU_EXIT=Menu.FIRST+5;

	private static final int DIALOG_ABOUT=1;

	public File mCurrentPath=null;

	public int mFirstFilePosition=0;

	private ArrayList<File> mFiles=new ArrayList<File>();

	private ArrayList<IconifiedText> mDirectoryEntries=new ArrayList<IconifiedText>();

	private Button mButtonPause;

	private ListView mFileList;

	private TextView mNowTime;
	private TextView mEndTime;

	private TextView mPath;
	private TextView mFile;

	private SeekBar mPositionSeekBar;

	private IBeRoXMService mIBeRoXMService=null;

	private SharedPreferences preferences;

	private boolean mLooping=true;

	private boolean mPause=true;
	private boolean mLoaded=false;

	private Boolean adsEnabled=true;

	private AdView adView=null;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);

		adsEnabled=true;

		try{
			PackageManager packageManager=getPackageManager();
			int signaturesMatch=packageManager.checkSignatures(getApplicationContext().getPackageName(),"com.bero.license.multimedia.beroxm");
			if(signaturesMatch==PackageManager.SIGNATURE_MATCH){
				adsEnabled=false;
				packageManager.setComponentEnabledSetting(new ComponentName("com.bero.license.multimedia.beroxm","com.bero.license.multimedia.beroxm.LicenseActivity"),PackageManager.COMPONENT_ENABLED_STATE_DISABLED,PackageManager.DONT_KILL_APP);
			}
		}catch(Exception e){
		}

		setContentView(R.layout.main);

		preferences=getSharedPreferences(TAG,0);

		adView=(AdView)findViewById(R.id.ad);

		AdManager.setTestDevices(new String[]{AdManager.TEST_EMULATOR,"70689742B111D50A1A1168CF3260C7E2","CCB8D61953C38CF83AAD8313517FF015","B90858F9AC6666D483A6C67A48B17DC6","F645442AC78AC18F9671D325E02DB1A3"});

		if(adView!=null){
			adView.setEnabled(adsEnabled);

			adView.setRequestInterval(60);
		}

		setContent();

		mFileList.setBackgroundResource(android.R.color.background_dark);

		mLooping=preferences.getBoolean("looping",mLooping);

		File filePath=new File(preferences.getString("path",android.os.Environment.getExternalStorageDirectory().getAbsolutePath()));
		if(!filePath.exists()){
			filePath=android.os.Environment.getExternalStorageDirectory();
		}
		setAdapter(filePath);

	}

	@Override
	public void onDestroy(){
		super.onDestroy();
	}

	@Override
	public void onStart(){
		super.onStart();
		if(showEula(this)){
		}
		if(adsEnabled){
			if(adView!=null){
				adView.setEnabled(true);

				adView.setVisibility(AdView.VISIBLE);

				AlphaAnimation animation=new AlphaAnimation(0.0f,1.0f);
				animation.setDuration(400);
				animation.setFillAfter(true);
				animation.setInterpolator(new AccelerateInterpolator());
				adView.startAnimation(animation);
			}
		}else{
			try{
				if(adView!=null){
					adView.setEnabled(false);

					adView.setVisibility(AdView.GONE);
				}
			}catch(Exception e){
			}
		}
	}

	@Override
	public void onResume(){
		if(mIBeRoXMService==null){
			startService(new Intent(MainActivity.this,BeRoXMService.class));
			bindService(new Intent(MainActivity.this,BeRoXMService.class),mConnection,Context.BIND_AUTO_CREATE);
		}else{
			if(adsEnabled){
				try{
					PackageManager packageManager=getPackageManager();
					int signaturesMatch=packageManager.checkSignatures(getApplicationContext().getPackageName(),"com.bero.license.multimedia.beroxm");
					if(signaturesMatch==PackageManager.SIGNATURE_MATCH){
						adsEnabled=false;
						packageManager.setComponentEnabledSetting(new ComponentName("com.bero.license.multimedia.beroxm","com.bero.license.multimedia.beroxm.LicenseActivity"),PackageManager.COMPONENT_ENABLED_STATE_DISABLED,PackageManager.DONT_KILL_APP);
						adView.setEnabled(false);
						adView.setVisibility(AdView.GONE);
					}
				}catch(Exception e){
				}
			}
		}

		super.onResume();
	}

	@Override
	public void onPause(){
		super.onPause();
		try{
			unbindService(mConnection);
		}catch(Exception e){
			Log.e(TAG,"Error disconnecting from BeRoXMService",e);
		}
	}

	public static int filetype(final File f){
		final String s=f.toString();
		if(f.isDirectory()){
			return 0;
		}else if(s.endsWith(".mod")||s.endsWith(".MOD")){
			return 1;
		}else if(s.endsWith(".wow")||s.endsWith(".WOW")){
			return 2;
		}else if(s.endsWith(".xm")||s.endsWith(".XM")){
			return 3;
		}else{
			return 1337;
		}
	}

	public void exit(){
		// System.exit(0);
		if(mIBeRoXMService!=null){
			try{
				mIBeRoXMService.stop();
			}catch(RemoteException e){
				e.printStackTrace();
			}
			stopService(new Intent(MainActivity.this,BeRoXMService.class));
			mIBeRoXMService=null;
		}
		try{
			Thread.sleep(1000);
		}catch(InterruptedException e){
		}
		android.os.Process.killProcess(android.os.Process.myPid());
	}

	@Override
	public boolean onKeyDown(int keyCode,KeyEvent event){
		switch(keyCode){
			case KeyEvent.KEYCODE_BACK:
				moveTaskToBack(true);
				return true;
			default:
				return super.onKeyDown(keyCode,event);
		}
	}

	@Override
	public int compare(File f1,File f2){
		int t1=filetype(f1),t2=filetype(f2);
		return (t1!=t2)?(t1-t2):f1.getName().toLowerCase().compareTo(f2.getName().toLowerCase());
	}

	private boolean setAdapter(final File fpath){
		if(fpath.isDirectory()){
			return setAdapterFromDirectory(fpath);
		}
		return false;
	}

	private class DirectoryData{
		File[] files;
		int countDirectories,countModules;
	}

	private DirectoryData parseDir(File filePath){

		int countDirectories=0,countModules=0;

		File[] files;
		try{
			files=filePath.listFiles();
		}catch(Exception e){
			return null;
		}

		if((files==null)||(files.length==0)){
			return null;
		}

		Comparator<File> cmp=this;
		Arrays.sort(files,cmp);

		for(int i=0;i<files.length;i++){
			switch(filetype(files[i])){
				case 0: {
					countDirectories++;
					break;
				}
				case 1:
				case 2:
				case 3: {
					countModules++;
					break;
				}
				default: {
					break;
				}
			}
		}

		final int countTotal=countDirectories+countModules;
		if(countTotal==0){
			return null;
		}

		DirectoryData r=new DirectoryData();
		r.countDirectories=countDirectories;
		r.countModules=countModules;
		r.files=new File[countDirectories+countModules];
		for(int i=0;i<countTotal;i++){
			r.files[i]=files[i];
		}
		return r;
	}

	private boolean setAdapterResultFromDirectory(final File filePath,final DirectoryData directoryData){
		try{

			String path=filePath.getAbsolutePath();
			mPath.setText(path+(path.endsWith(File.separator)?"":File.separator));

			preferences.edit().putString("path",path).commit();

			File[] files;
			int countDirectories,countModules;

			if(directoryData==null){
				files=null;
				countDirectories=0;
				countModules=0;
			}else{
				files=directoryData.files;
				countDirectories=directoryData.countDirectories;
				countModules=directoryData.countModules;
			}

			final int countTotal=countDirectories+countModules;

			mCurrentPath=filePath;
			mFirstFilePosition=countDirectories;

			mFiles.clear();
			mDirectoryEntries.clear();

			Drawable dir_icon=getResources().getDrawable(R.drawable.folder);
			Drawable aud_icon=getResources().getDrawable(R.drawable.audio);

			String currentPathString=mCurrentPath.getAbsolutePath();

			int pathLength=currentPathString.length();
			if(!currentPathString.equals(File.separator)){
				pathLength++;
				mDirectoryEntries.add(new IconifiedText("..",dir_icon));
				mFiles.add(new File(currentPathString.substring(0,currentPathString.lastIndexOf(File.separator)+1)));
			}
			for(int i=0;i<countTotal;i++){
				File file=files[i];
				String fileName=file.getName();
				if(i<countDirectories){
					mDirectoryEntries.add(new IconifiedText(fileName,dir_icon));
				}else{
					mDirectoryEntries.add(new IconifiedText(fileName,aud_icon));
				}
				mFiles.add(file);
			}

			IconifiedTextListAdapter ita=new IconifiedTextListAdapter(this);
			ita.setListItems(mDirectoryEntries);
			mFileList.setAdapter(ita);
			return true;
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}

	}

	private boolean setAdapterFromDirectory(final File filePath){
		try{
			/*
			 * mFiles.clear(); mDirectoryEntries.clear();
			 * 
			 * String currentPathString=filePath.getAbsolutePath();
			 * 
			 * if(!currentPathString.equals(File.separator)){ Drawable dir_icon=getResources().getDrawable(R.drawable.folder); mDirectoryEntries.add(new IconifiedText("..",dir_icon)); mFiles.add(new File(currentPathString.substring(0,currentPathString.lastIndexOf(File.separator)+1))); }
			 * 
			 * IconifiedTextListAdapter ita=new IconifiedTextListAdapter(this); ita.setListItems(mDirectoryEntries); mFileList.setAdapter(ita);
			 */

			new Thread(){
				public void run(){
					final DirectoryData directoryData=parseDir(filePath);

					runOnUiThread(new Runnable(){
						public void run(){
							setAdapterResultFromDirectory(filePath,directoryData);
						}
					});

				}
			}.start();

			return true;
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}
	}

	private void setContent(){
		mPath=(TextView)findViewById(R.id.path);
		mFile=(TextView)findViewById(R.id.file);

		mButtonPause=(Button)findViewById(R.id.ButtonPause);

		mFileList=(ListView)findViewById(R.id.FileList);

		mNowTime=(TextView)findViewById(R.id.nowTime);
		mEndTime=(TextView)findViewById(R.id.endTime);

		mPositionSeekBar=(SeekBar)findViewById(R.id.PositionSeekBar);

		mPositionSeekBar.setOnSeekBarChangeListener(onPositionSeekBar);

		mButtonPause.setOnClickListener(onButtonPause);

		mFileList.setOnItemClickListener(onSelectItem);
		mFileList.setOnItemLongClickListener(onPressItem);

	}

	View.OnClickListener onButtonPause=new View.OnClickListener(){
		public void onClick(View v){
			if(mLoaded){
				if(mPause){
					mPause=false;
					try{
						if(mIBeRoXMService!=null){
							mIBeRoXMService.resume();
						}
					}catch(RemoteException e){
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_pause));
				}else{
					mPause=true;
					try{
						if(mIBeRoXMService!=null){
							mIBeRoXMService.pause();
						}
					}catch(RemoteException e){
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_play));
				}
			}
		}
	};

	private int realProgress=0;
	private long realDuration=0;

	Runnable updateNow=new Runnable(){

		@Override
		public void run(){
			mNowTime.setText((realProgress<3600)?String.format("%d:%02d",realProgress/60,realProgress%60):String.format("%d:%02d:%02d",realProgress/3600,(realProgress%3600)/60,realProgress%60));
			mPositionSeekBar.setProgress(realProgress);
		}

	};

	Runnable updateDuration=new Runnable(){

		@Override
		public void run(){
			int duration=(int)((realDuration+22500)/44100);
			mEndTime.setText((duration<3600)?String.format("%d:%02d",duration/60,duration%60):String.format("%d:%02d:%02d",duration/3600,(duration%3600)/60,duration%60));
			mPositionSeekBar.setMax(duration);
		}

	};

	OnSeekBarChangeListener onPositionSeekBar=new OnSeekBarChangeListener(){

		public void onProgressChanged(SeekBar seekBar,int progress,boolean fromUser){
			if(fromUser){
				mNowTime.setText((progress<3600)?String.format("%d:%02d",progress/60,progress%60):String.format("%d:%02d:%02d",progress/3600,(progress%3600)/60,progress%60));
			}
		}

		public void onStartTrackingTouch(SeekBar seekBar){

		}

		public void onStopTrackingTouch(SeekBar seekBar){
			try{
				mIBeRoXMService.seek(seekBar.getProgress()*44100);
			}catch(RemoteException e){ // TODO Auto-generated catch block e.printStackTrace(); }
			}
		}
	};

	AdapterView.OnItemLongClickListener onPressItem=new AdapterView.OnItemLongClickListener(){
		public boolean onItemLongClick(AdapterView<?> a,View v,int i,long k){
			return false;
		}
	};

	AdapterView.OnItemClickListener onSelectItem=new AdapterView.OnItemClickListener(){
		public void onItemClick(AdapterView<?> a,View v,int i,long k){
			if(a==null){
				return;
			}
			if((k>=0)&&(k<mFiles.size())){
				File file=mFiles.get((int)k);
				if(file.exists()){
					if(file.isDirectory()){
						setAdapter(file);
					}else{
						mFile.setText(file.getName());
						if(mIBeRoXMService!=null){
							try{
								if(mIBeRoXMService.load(file.getCanonicalPath())){
									mIBeRoXMService.setLooping(mLooping);
									mIBeRoXMService.play();
									mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_pause));
									mPause=false;
									mLoaded=true;
								}else{
									mLoaded=false;
									mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_play));
									mPause=true;
								}
							}catch(RemoteException e){
								e.printStackTrace();
							}catch(IOException e){
								e.printStackTrace();
							}
						}
					}
				}
			}
		}
	};

	private ServiceConnection mConnection=new ServiceConnection(){
		public void onServiceConnected(ComponentName className,IBinder service){
			mIBeRoXMService=IBeRoXMService.Stub.asInterface((IBinder)service);
			try{
				Log.d(TAG,"Registering callback...");
				mIBeRoXMService.registerCallback(mCallback);
				if(mIBeRoXMService!=null){
					mFile.setText(mIBeRoXMService.getTitle());
					mLoaded=mIBeRoXMService.loaded();
					mPause=mIBeRoXMService.paused();
					if(mPause||!mLoaded){
						mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_play));
					}else{
						mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_pause));
					}
				}
			}catch(Exception e){
				Log.e(TAG,"Error registering callback!",e);
			}
		}

		public void onServiceDisconnected(ComponentName className){
			try{
				Log.d(TAG,"Unregistering callback...");
				mIBeRoXMService.unregisterCallback(mCallback);
			}catch(Exception e){
				Log.e(TAG,"Error unregistering callback!",e);
			}
			mIBeRoXMService=null;
		}
	};

	private IBeRoXMServiceCallback mCallback=new IBeRoXMServiceCallback.Stub(){

		@Override
		public void durationChanged(long currentDuration) throws RemoteException{
			realDuration=currentDuration;
			runOnUiThread(updateDuration);
		}

		@Override
		public void positionChanged(long currentPosition) throws RemoteException{
			realProgress=(int)(currentPosition);
			runOnUiThread(updateNow);
		}

		@Override
		public void titleChanged(String currentTitle) throws RemoteException{
			if((currentTitle!=null)&&(!currentTitle.trim().equals(""))){
				mFile.setText(currentTitle);
			}
		}

		@Override
		public void stateChanged(boolean pause) throws RemoteException{
			mPause=pause;
			if(pause){
				mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_play));
			}else{
				mButtonPause.setBackgroundDrawable(getResources().getDrawable(R.drawable.s_pause));
			}
		}

	};

	@Override
	public boolean onCreateOptionsMenu(Menu menu){
		super.onCreateOptionsMenu(menu);

		menu.add(0,MENU_LOOPING,0,mLooping?R.string.menu_looping_off:R.string.menu_looping_on).setIcon(R.drawable.looping).setShortcut('0','l');

		menu.add(0,MENU_REFRESH,0,R.string.menu_refresh).setIcon(R.drawable.refresh).setShortcut('0','r');

		if(adsEnabled){
			menu.add(0,MENU_BUY,0,R.string.menu_buy).setIcon(R.drawable.buy).setShortcut('0','d');
		}

		menu.add(0,MENU_ABOUT,0,R.string.menu_about).setIcon(R.drawable.about).setShortcut('0','a');

		menu.add(0,MENU_EXIT,0,R.string.menu_exit).setIcon(R.drawable.exit).setShortcut('0','e');

		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu){
		super.onPrepareOptionsMenu(menu);

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item){
		switch(item.getItemId()){
			case MENU_LOOPING: {
				mLooping=!mLooping;
				preferences.edit().putBoolean("looping",mLooping).commit();
				if(mIBeRoXMService!=null){
					try{
						mIBeRoXMService.setLooping(mLooping);
					}catch(RemoteException e){
						e.printStackTrace();
					}
				}
				if(mLooping){
					item.setTitle(R.string.menu_looping_off);
				}else{
					item.setTitle(R.string.menu_looping_on);
				}
				return true;
			}
			case MENU_REFRESH: {
				setAdapter(mCurrentPath);
				return true;
			}
			case MENU_BUY: {
				startActivity(new Intent(Intent.ACTION_VIEW,Uri.parse("market://details?id=com.bero.license.multimedia.beroxm")));
				return true;
			}
			case MENU_ABOUT: {
				showDialog(DIALOG_ABOUT);
				return true;
			}
			case MENU_EXIT: {
				exit();
				return true;
			}
		}
		return super.onOptionsItemSelected(item);

	}

	@Override
	protected Dialog onCreateDialog(int id){

		switch(id){

			case DIALOG_ABOUT: {
				View view=LayoutInflater.from(this).inflate(R.layout.about,null);
				/*
				 * TextView tv=(TextView)view.findViewById(R.id.uid); tv.setText("Device unique ID:\n\n"+uid+"\n\n");
				 */
				return new AlertDialog.Builder(this).setTitle(Utils.getApplicationName(this)+" v"+Utils.getVersionNumber(this)).setView(view).setCancelable(true).setPositiveButton(R.string.about_ok,null).setNeutralButton(R.string.about_web,new OnClickListener(){

					public void onClick(DialogInterface dialog,int which){
						startActivity(new Intent(Intent.ACTION_VIEW,Uri.parse("http://www.rosseaux.com/")));
					}

				}).setNegativeButton(R.string.about_email,new OnClickListener(){

					public void onClick(DialogInterface dialog,int which){
						startActivity(new Intent(Intent.ACTION_SENDTO,Uri.parse("mailto:benjamin@rosseaux.com")));
					}

				}).create();
			}

		}
		return null;

	}

	@Override
	protected void onPrepareDialog(int id,Dialog dialog){
		super.onPrepareDialog(id,dialog);

		switch(id){
			case DIALOG_ABOUT: {
				break;
			}
		}
	}

	private static final String PREFERENCE_EULA_ACCEPTED="eula.accepted";
	private static final String PREFERENCES_EULA="eula";

	private static boolean showEula(final MainActivity activity){
		final SharedPreferences preferences=activity.getSharedPreferences(PREFERENCES_EULA,Activity.MODE_PRIVATE);
		if(!preferences.getBoolean(PREFERENCE_EULA_ACCEPTED,false)){
			final AlertDialog.Builder builder=new AlertDialog.Builder(activity);
			builder.setTitle(R.string.eula_title);
			builder.setCancelable(true);
			builder.setPositiveButton(R.string.eula_accept,new DialogInterface.OnClickListener(){
				public void onClick(DialogInterface dialog,int which){
					preferences.edit().putBoolean(PREFERENCE_EULA_ACCEPTED,true).commit();
				}
			});
			builder.setNegativeButton(R.string.eula_refuse,new DialogInterface.OnClickListener(){
				public void onClick(DialogInterface dialog,int which){
					activity.finish();
				}
			});
			builder.setOnCancelListener(new DialogInterface.OnCancelListener(){
				public void onCancel(DialogInterface dialog){
					activity.finish();
				}
			});

			builder.setMessage(readFile(activity,R.raw.eula));
			builder.create().show();
			return(false);
		}else{
			return(true);
		}
	}

	private static CharSequence readFile(Activity activity,int id){
		BufferedReader in=null;
		try{
			in=new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(id)));
			String line;
			StringBuilder buffer=new StringBuilder();
			while((line=in.readLine())!=null)
				buffer.append(line).append('\n');
			return buffer;
		}catch(IOException e){
			return "";
		}finally{
			closeStream(in);
		}
	}

	private static void closeStream(Closeable stream){
		if(stream!=null){
			try{
				stream.close();
			}catch(IOException e){
			}
		}
	}

}