package com.bero.multimedia.beroxm;

import java.io.File;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import java.util.List;

import java.math.BigInteger;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.provider.MediaStore.Audio;
import android.provider.MediaStore.Video;
import android.util.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

@SuppressWarnings("unused")
public class Utils{

	public static boolean isIntentAvailable(final Context context,final Intent intent){
		final PackageManager packageManager=context.getPackageManager();
		List<ResolveInfo> list=packageManager.queryIntentActivities(intent,PackageManager.MATCH_DEFAULT_ONLY);
		return list.size()>0;
	}

	public static String getVersionNumber(Context context){
		String version="?";
		try{
			PackageInfo pi=context.getPackageManager().getPackageInfo(context.getPackageName(),0);
			version=pi.versionName;
		}catch(PackageManager.NameNotFoundException e){
			Log.e(Globals.TAG,"Package name not found",e);
		};
		return version;
	}

	public static String getApplicationName(Context context){
		String name="?";
		try{
			PackageInfo pi=context.getPackageManager().getPackageInfo(context.getPackageName(),0);
			name=context.getString(pi.applicationInfo.labelRes);
		}catch(PackageManager.NameNotFoundException e){
			Log.e(Globals.TAG,"Package name not found",e);
		};
		return name;
	}

	public static String getMimeType(String filename){
		return "application/binary";
	}

	public static boolean isVideo(String filename){
		String mimeType=getMimeType(filename);
		if(mimeType!=null&&mimeType.startsWith("video/")){
			return true;
		}else{
			return false;
		}
	}

	public static boolean isLocal(String uri){
		return((uri!=null)&&!(uri.startsWith("http://")||uri.startsWith("https://")));
	}

	public static String getExtension(String uri){
		if(uri!=null){
			int dot=uri.lastIndexOf(".");
			return (dot>=0)?uri.substring(dot):"";
		}else{
			return null;
		}
	}

	public static boolean isMediaUri(String uri){
		return((uri.startsWith(Audio.Media.INTERNAL_CONTENT_URI.toString())||uri.startsWith(Audio.Media.EXTERNAL_CONTENT_URI.toString())||uri.startsWith(Video.Media.INTERNAL_CONTENT_URI.toString())||uri.startsWith(Video.Media.EXTERNAL_CONTENT_URI.toString())));
	}

	public static Uri getUri(File file){
		return (file!=null)?Uri.fromFile(file):null;
	}

	public static File getFile(Uri uri){
		if(uri!=null){
			String filepath=uri.getPath();
			return (filepath!=null)?new File(filepath):null;
		}else{
			return null;
		}
	}

	public static File getPathWithoutFilename(File file){
		if(file!=null){
			if(file.isDirectory()){
				return file;
			}else{
				String filename=file.getName();
				String filepath=file.getAbsolutePath();
				String pathwithoutname=filepath.substring(0,filepath.length()-filename.length());
				if(pathwithoutname.endsWith("/")){
					pathwithoutname=pathwithoutname.substring(0,pathwithoutname.length()-1);
				}
				return new File(pathwithoutname);
			}
		}else{
			return null;
		}
	}

	public static File getFile(String curdir,String file){
		String separator="/";
		if(curdir.endsWith("/")){
			separator="";
		}
		File clickedFile=new File(curdir+separator+file);
		return clickedFile;
	}

	public static File getFile(File curdir,String file){
		return getFile(curdir.getAbsolutePath(),file);
	}

	public static String getMD5Hash(String input){
		try{
			MessageDigest md=MessageDigest.getInstance("MD5");
			byte[] messageDigest=md.digest(input.getBytes());
			BigInteger number=new BigInteger(1,messageDigest);
			String md5=number.toString(16);
			while(md5.length()<32){
				md5="0"+md5;
			}
			return md5;
		}catch(NoSuchAlgorithmException e){
			Log.e(Globals.TAG,e.getMessage());
			return null;
		}

	}

	public static String getTextFile(File file){
		String s=new String();
		try{
			FileInputStream fileInputStream=new FileInputStream(file);
			BufferedReader buf=new BufferedReader(new InputStreamReader(fileInputStream));
			String readString=new String();
			while((readString=buf.readLine())!=null){
				s=s+readString+"\n";
			}
			if(buf!=null){
				try{
					buf.close();
				}catch(IOException e){
				}catch(Exception e){
				}
			}
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}catch(IOException e){
			e.printStackTrace();
		}
		return s;
	}
	
	public static Drawable getDrawableForMimetype(Context context,String mimetype){
		PackageManager pm=context.getPackageManager();

		Intent intent=new Intent(Intent.ACTION_VIEW);
		intent.setType(mimetype);

		final List<ResolveInfo> lri=pm.queryIntentActivities(intent,PackageManager.MATCH_DEFAULT_ONLY);

		if((lri!=null)&&(lri.size()>0)){
			final ResolveInfo ri=lri.get(0);
			Drawable d=ri.loadIcon(pm);
			if(d!=null){
				return d;
			}
		}

		return null;
	}	
	
	public static String getNameForMimetype(Context context,String mimetype){
		PackageManager pm=context.getPackageManager();

		Intent intent=new Intent(Intent.ACTION_VIEW);
		intent.setType(mimetype);

		final List<ResolveInfo> lri=pm.queryIntentActivities(intent,PackageManager.MATCH_DEFAULT_ONLY);

		if((lri!=null)&&(lri.size()>0)){
			final ResolveInfo ri=lri.get(0);
			try{
				return ri.activityInfo.packageName;
			}catch(Exception e){				
				return null;
			}
		}

		return null;
	}	

}
